import { WEBGLRenderbuffer } from '@luma.gl/webgl';
import { equals } from '@math.gl/core';
import { _deepEqual as deepEqual } from '@deck.gl/core';
import CollisionFilterPass from './collision-filter-pass';
import MaskEffect from '../mask/mask-effect';
const DOWNSCALE = 2;
export default class CollisionFilterEffect {
  constructor() {
    this.id = 'collision-filter-effect';
    this.props = null;
    this.useInPicking = true;
    this.order = 1;
    this.channels = {};
    this.collisionFilterPass = void 0;
    this.collisionFBOs = {};
    this.dummyCollisionMap = void 0;
    this.lastViewport = void 0;
  }

  preRender(device, {
    effects: allEffects,
    layers,
    layerFilter,
    viewports,
    onViewportActive,
    views,
    isPicking,
    preRenderStats = {}
  }) {
    var _preRenderStats$mask;

    if (!this.dummyCollisionMap) {
      this.dummyCollisionMap = device.createTexture({
        width: 1,
        height: 1
      });
    }

    if (isPicking) {
      return;
    }

    const collisionLayers = layers.filter(({
      props: {
        visible,
        collisionEnabled
      }
    }) => visible && collisionEnabled);

    if (collisionLayers.length === 0) {
      this.channels = {};
      return;
    }

    if (!this.collisionFilterPass) {
      this.collisionFilterPass = new CollisionFilterPass(device, {
        id: 'default-collision-filter'
      });
    }

    const effects = allEffects === null || allEffects === void 0 ? void 0 : allEffects.filter(e => e.constructor === MaskEffect);
    const maskEffectRendered = (_preRenderStats$mask = preRenderStats['mask-effect']) === null || _preRenderStats$mask === void 0 ? void 0 : _preRenderStats$mask.didRender;

    const channels = this._groupByCollisionGroup(device, collisionLayers);

    const viewport = viewports[0];
    const viewportChanged = !this.lastViewport || !this.lastViewport.equals(viewport) || maskEffectRendered;

    for (const collisionGroup in channels) {
      const collisionFBO = this.collisionFBOs[collisionGroup];
      const renderInfo = channels[collisionGroup];
      const [width, height] = device.canvasContext.getPixelSize();
      collisionFBO.resize({
        width: width / DOWNSCALE,
        height: height / DOWNSCALE
      });

      this._render(renderInfo, {
        effects,
        layerFilter,
        onViewportActive,
        views,
        viewport,
        viewportChanged
      });
    }
  }

  _render(renderInfo, {
    effects,
    layerFilter,
    onViewportActive,
    views,
    viewport,
    viewportChanged
  }) {
    const {
      collisionGroup
    } = renderInfo;
    const oldRenderInfo = this.channels[collisionGroup];

    if (!oldRenderInfo) {
      return;
    }

    const needsRender = viewportChanged || renderInfo === oldRenderInfo || !deepEqual(oldRenderInfo.layers, renderInfo.layers, 1) || renderInfo.layerBounds.some((b, i) => !equals(b, oldRenderInfo.layerBounds[i])) || renderInfo.allLayersLoaded !== oldRenderInfo.allLayersLoaded || renderInfo.layers.some(layer => layer.props.transitions);
    this.channels[collisionGroup] = renderInfo;

    if (needsRender) {
      this.lastViewport = viewport;
      const collisionFBO = this.collisionFBOs[collisionGroup];
      this.collisionFilterPass.renderCollisionMap(collisionFBO, {
        pass: 'collision-filter',
        isPicking: true,
        layers: renderInfo.layers,
        effects,
        layerFilter,
        viewports: viewport ? [viewport] : [],
        onViewportActive,
        views,
        moduleParameters: {
          dummyCollisionMap: this.dummyCollisionMap,
          devicePixelRatio: collisionFBO.device.canvasContext.getDevicePixelRatio() / DOWNSCALE
        }
      });
    }
  }

  _groupByCollisionGroup(device, collisionLayers) {
    const channelMap = {};

    for (const layer of collisionLayers) {
      const {
        collisionGroup
      } = layer.props;
      let channelInfo = channelMap[collisionGroup];

      if (!channelInfo) {
        channelInfo = {
          collisionGroup,
          layers: [],
          layerBounds: [],
          allLayersLoaded: true
        };
        channelMap[collisionGroup] = channelInfo;
      }

      channelInfo.layers.push(layer);
      channelInfo.layerBounds.push(layer.getBounds());

      if (!layer.isLoaded) {
        channelInfo.allLayersLoaded = false;
      }
    }

    for (const collisionGroup of Object.keys(channelMap)) {
      if (!this.collisionFBOs[collisionGroup]) {
        this.createFBO(device, collisionGroup);
      }

      if (!this.channels[collisionGroup]) {
        this.channels[collisionGroup] = channelMap[collisionGroup];
      }
    }

    for (const collisionGroup of Object.keys(this.collisionFBOs)) {
      if (!channelMap[collisionGroup]) {
        this.destroyFBO(collisionGroup);
      }
    }

    return channelMap;
  }

  getModuleParameters(layer) {
    const {
      collisionGroup
    } = layer.props;
    const {
      collisionFBOs,
      dummyCollisionMap
    } = this;
    return {
      collisionFBO: collisionFBOs[collisionGroup],
      dummyCollisionMap: dummyCollisionMap
    };
  }

  cleanup() {
    if (this.dummyCollisionMap) {
      this.dummyCollisionMap.delete();
      this.dummyCollisionMap = undefined;
    }

    this.channels = {};

    for (const collisionGroup of Object.keys(this.collisionFBOs)) {
      this.destroyFBO(collisionGroup);
    }

    this.collisionFBOs = {};
    this.lastViewport = undefined;
  }

  createFBO(device, collisionGroup) {
    const {
      width,
      height
    } = device.gl.canvas;
    const collisionMap = device.createTexture({
      format: 'rgba8unorm',
      width,
      height,
      sampler: {
        minFilter: 'nearest',
        magFilter: 'nearest',
        addressModeU: 'clamp-to-edge',
        addressModeV: 'clamp-to-edge'
      }
    });
    const depthBuffer = new WEBGLRenderbuffer(device, {
      format: 'depth16unorm',
      width,
      height
    });
    this.collisionFBOs[collisionGroup] = device.createFramebuffer({
      id: `collision-${collisionGroup}`,
      width,
      height,
      colorAttachments: [collisionMap],
      depthStencilAttachment: depthBuffer
    });
  }

  destroyFBO(collisionGroup) {
    var _fbo$colorAttachments, _fbo$depthStencilAtta;

    const fbo = this.collisionFBOs[collisionGroup];
    (_fbo$colorAttachments = fbo.colorAttachments[0]) === null || _fbo$colorAttachments === void 0 ? void 0 : _fbo$colorAttachments.destroy();
    (_fbo$depthStencilAtta = fbo.depthStencilAttachment) === null || _fbo$depthStencilAtta === void 0 ? void 0 : _fbo$depthStencilAtta.destroy();
    fbo.destroy();
    delete this.collisionFBOs[collisionGroup];
  }

}
//# sourceMappingURL=collision-filter-effect.js.map